; *****************************************************************************
; * ZOOMER by Mr_Mungo <magnus82@hotmail.com> for the IRCNet #asm competition *
; *                                                                           *
; * assemble with NASM                                    COM size: 209 bytes *
; *****************************************************************************

[BITS 16]
[ORG 0x100]

CMDLINE_LEN equ 0x80
CMDLINE_START equ 0x81

start:
  ; Switch into graphics mode 13h
  mov ax,0x0013
  int 0x10
  mov ax,0xA000
  mov es,ax
  ; Null-terminate the command line
  mov bl,byte [CMDLINE_LEN]
  xor bh,bh
  mov byte [CMDLINE_START+bx],0
  ; Open the file specified on the cmdline
  mov dx,CMDLINE_START+1
  mov ah,0x3D ; AX=0x3D00
  int 0x21
  ; Read the data
  mov bx,ax
  mov ah,0x3F
  mov CX,40768 ; Size of palette + image
  mov dx,palette
  int 0x21
  ; Set the palette
  mov dx,0x3C8
  xor al,al
  out dx,al
  inc dx
  mov cx,768
  mov si,palette
  cld
  rep outsb
  ; Draw!!!
  mov ax, 100 ;Half the size
  push ax    ;Keep the size on the stack
frame_start:
  mov di,32160 ; 320*100+160 = middle of screen
  sub di,ax
  mov bx,320
  mul bx
  sub di,ax
  ; DI is now the position of the top-left pixel
  ; Clear 321 bytes above DI to remove the last frame
  xor al,al
  mov cx,321
  sub di,cx
  rep stosb
  ; Restore AX
  pop ax
  push ax
  ; Double AX so we get the true size
  shl ax,1
  ; BH is the error variable for columns, CH is the error variable for lines
  mov bx,200 ; Clear BH at the same time
  sub bx,ax ; BL=200-ax=error-increase per pixel or line
  mov si,pix
  xor ch,ch
 newline:
  xor cl,cl ; CL = Number of pixels drawn this line
  cmp si,pix_end ; Check if we have reached EOL
  jae frame_end
  es mov byte [di],0 ; Remove the pixel before the first column
  inc di
donext:
  MOVSB
  inc cl
  add bh,bl ; Update the line error variable
test_error:
  cmp bh,al ; Check if BH has overflown, i e if we should skip a pixel
  jb error_fixed
fix_error:
  sub bh,al
  inc si
  jmp test_error
error_fixed:
  cmp cl,al
  jb donext
  es mov byte [di],0 ; Remove the pixel after the last column
  add di,319
  sub di,ax
  add ch,bl
test_line_error:
  ; Check if we should skip a line
  cmp ch,al
  jb newline
  add si,200
  sub ch,al
  jmp test_line_error
frame_end:
  ; Remove the line below this
  mov cx,321
  xor al,al
  rep stosb
  ; Wait for vrt
  MOV DX,0x3DA
sync_out:
  IN AL,DX
  AND AL,8
  JNZ sync_out
sync_in:
  IN AL,DX
  AND AL,8
  JZ sync_in
  ; Check for keypress
  mov dl,0xFF
  mov ah,0x06
  int 0x21
  jnz exit
  ; Update the size
  pop ax
  add al, byte [direction]
  cmp al,100
  je change_direction
  cmp al,25
  jne continue
change_direction:
  neg byte [direction]
continue:
  push ax
  jmp frame_start

exit:
 mov ax,0x0003
 int 0x10
 int 0x20

direction db -1

; Some uninitialized storage
palette EQU $
pix EQU $+768
pix_end EQU $+768+40000
